@@ -173,12 +173,12 @@ int flb_tail_file_pack_line(msgpack_sbuffer *mp_sbuf, msgpack_packer *mp_pck,
     return 0;
 }
 
-static int process_content(struct flb_tail_file *file, off_t *bytes)
+static int process_content(struct flb_tail_file *file, size_t *bytes)
 {
-    int len;
+    size_t len;
     int lines = 0;
     int ret;
-    off_t processed_bytes = 0;
+    size_t processed_bytes = 0;
     char *data;
     char *end;
     char *p;
@@ -197,7 +197,7 @@ static int process_content(struct flb_tail_file *file, off_t *bytes)
     msgpack_packer *out_pck;
     struct flb_tail_config *ctx = file->config;
 
-    /* Create a temporal msgpack buffer */
+    /* Create a temporary msgpack buffer */
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
     out_sbuf = &mp_sbuf;
@@ -377,7 +377,7 @@ static int tag_compose(char *tag, char *fname, char *out_buf, size_t *out_size,
 #endif
 {
     int i;
-    int len;
+    size_t len;
     char *p;
     size_t buf_s = 0;
 #ifdef FLB_HAVE_REGEX
@@ -539,16 +539,70 @@ static inline int flb_tail_file_exists(struct stat *st,
     return FLB_FALSE;
 }
 
+/*
+ * Based in the configuration or database offset, set the proper 'offset' for the
+ * file in question.
+ */
+static int set_file_position(struct flb_tail_config *ctx,
+                             struct flb_tail_file *file)
+{
+    int64_t ret;
+
+#ifdef FLB_HAVE_SQLDB
+    /*
+     * If the database option is enabled, try to gather the file position. The
+     * database function updates the file->offset entry.
+     */
+    if (ctx->db) {
+        ret = flb_tail_db_file_set(file, ctx);
+        if (ret == 0) {
+            if (file->offset > 0) {
+                ret = lseek(file->fd, file->offset, SEEK_SET);
+                if (ret == -1) {
+                    flb_errno();
+                    return -1;
+                }
+            }
+            else if (ctx->read_from_head == FLB_FALSE) {
+                ret = lseek(file->fd, 0, SEEK_END);
+                if (ret == -1) {
+                    flb_errno();
+                    return -1;
+                }
+                file->offset = ret;
+                flb_tail_db_file_offset(file, ctx);
+            }
+            return 0;
+        }
+    }
+#endif
+
+    if (ctx->read_from_head == FLB_TRUE) {
+        /* no need to seek, offset position is already zero */
+        return 0;
+    }
+
+    /* tail... */
+    ret = lseek(file->fd, 0, SEEK_END);
+    if (ret == -1) {
+        flb_errno();
+        return -1;
+    }
+    file->offset = ret;
+
+    return 0;
+}
+
 int flb_tail_file_append(char *path, struct stat *st, int mode,
                          struct flb_tail_config *ctx)
 {
     int fd;
     int ret;
-    int len;
-    off_t offset;
+    size_t len;
     char *tag;
     size_t tag_len;
     struct flb_tail_file *file;
+    struct stat lst;
 
     if (!S_ISREG(st->st_mode)) {
         return -1;
@@ -576,7 +630,6 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
     file->fd        = fd;
 
     /* On non-windows environments check if the original path is a link */
-    struct stat lst;
     ret = lstat(path, &lst);
     if (ret == 0) {
         if (S_ISLNK(lst.st_mode)) {
@@ -622,6 +675,7 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
     file->dmode_complete = true;
     file->dmode_buf = flb_sds_create_size(ctx->docker_mode == FLB_TRUE ? 65536 : 0);
     file->dmode_lastline = flb_sds_create_size(ctx->docker_mode == FLB_TRUE ? 20000 : 0);
+    file->dmode_firstline = false;
 #ifdef FLB_HAVE_SQLDB
     file->db_id     = 0;
 #endif
@@ -685,35 +739,23 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
         }
     }
 
-    /*
-     * Register or update the file entry, likely if the entry already exists
-     * into the database, the offset may be updated.
-     */
-#ifdef FLB_HAVE_SQLDB
-    if (ctx->db) {
-        flb_tail_db_file_set(file, ctx);
-    }
-#endif
-
-    /* Seek if required */
-    if (file->offset > 0) {
-        flb_plg_debug(ctx->ins, "inode=%"PRIu64" appended file following on offset=%lu",
-                      file->inode, file->offset);
-        offset = lseek(file->fd, file->offset, SEEK_SET);
-        if (offset == -1) {
-            flb_errno();
-            flb_tail_file_remove(file);
-            goto error;
-        }
+    /* Set the file position (database offset, head or tail) */
+    ret = set_file_position(ctx, file);
+    if (ret == -1) {
+        flb_tail_file_remove(file);
+        goto error;
     }
 
+    /* Remaining bytes to read */
     file->pending_bytes = file->size - file->offset;
 
 #ifdef FLB_HAVE_METRICS
     flb_metrics_sum(FLB_TAIL_METRIC_F_OPENED, 1, ctx->ins->metrics);
 #endif
 
-    flb_plg_debug(ctx->ins, "inode=%"PRIu64" appended as %s", file->inode, path);
+    flb_plg_debug(ctx->ins,
+                  "inode=%"PRIu64" with offset=%"PRId64" appended as %s",
+                  file->inode, file->offset, path);
     return 0;
 
 error:
@@ -757,7 +799,10 @@ void flb_tail_file_remove(struct flb_tail_file *file)
     flb_sds_destroy(file->dmode_lastline);
     mk_list_del(&file->_head);
     flb_tail_fs_remove(file);
-    close(file->fd);
+    /* avoid deleting file with -1 fd */
+    if (file->fd != -1) {
+        close(file->fd);
+    }
     if (file->tag_buf) {
         flb_free(file->tag_buf);
     }
@@ -795,13 +840,53 @@ int flb_tail_file_remove_all(struct flb_tail_config *ctx)
     return count;
 }
 
+static int adjust_counters(struct flb_tail_config *ctx, struct flb_tail_file *file)
+{
+    int ret;
+    int64_t offset;
+    struct stat st;
+
+    ret = fstat(file->fd, &st);
+    if (ret == -1) {
+        flb_errno();
+        return FLB_TAIL_ERROR;
+    }
+
+    /* Check if the file was truncated */
+    if (file->offset > st.st_size) {
+        offset = lseek(file->fd, 0, SEEK_SET);
+        if (offset == -1) {
+            flb_errno();
+            return FLB_TAIL_ERROR;
+        }
+
+        flb_plg_debug(ctx->ins, "inode=%"PRIu64" file truncated %s",
+                      file->inode, file->name);
+        file->offset = offset;
+        file->buf_len = 0;
+
+        /* Update offset in the database file */
+#ifdef FLB_HAVE_SQLDB
+        if (ctx->db) {
+            flb_tail_db_file_offset(file, ctx);
+        }
+#endif
+    }
+    else {
+        file->size = st.st_size;
+        file->pending_bytes = (st.st_size - file->offset);
+    }
+
+    return FLB_TAIL_OK;
+}
+
 int flb_tail_file_chunk(struct flb_tail_file *file)
 {
     int ret;
     char *tmp;
     size_t size;
-    off_t capacity;
-    off_t processed_bytes;
+    size_t capacity;
+    size_t processed_bytes;
     ssize_t bytes;
     struct stat st;
     struct flb_tail_config *ctx;
@@ -900,23 +985,21 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
             return FLB_TAIL_ERROR;
         }
         else {
-            file->size = st.st_size;
-            file->pending_bytes = (st.st_size - file->offset);
+            /* adjust file counters, returns FLB_TAIL_OK or FLB_TAIL_ERROR */
+            ret = adjust_counters(ctx, file);
         }
         /* Data was consumed but likely some bytes still remain */
-        return FLB_TAIL_OK;
+        return ret;
     }
     else if (bytes == 0) {
         /* We reached the end of file, let's wait for some incoming data */
-        ret = fstat(file->fd, &st);
-        if (ret == -1) {
-            flb_errno();
+        ret = adjust_counters(ctx, file);
+        if (ret == FLB_TAIL_OK) {
+            return FLB_TAIL_WAIT;
+        }
+        else {
             return FLB_TAIL_ERROR;
         }
-        file->size = st.st_size;
-        file->pending_bytes = (st.st_size - file->offset);
-
-        return FLB_TAIL_WAIT;
     }
     else {
         /* error */
@@ -1057,7 +1140,7 @@ char *flb_tail_file_name(struct flb_tail_file *file)
     char tmp[128];
 #elif defined(__APPLE__)
     char path[PATH_MAX];
-#elif defined(_MSC_VER)
+#elif defined(FLB_SYSTEM_WINDOWS)
     HANDLE h;
 #endif
 
@@ -1097,10 +1180,10 @@ char *flb_tail_file_name(struct flb_tail_file *file)
     memcpy(buf, path, len);
     buf[len] = '\0';
 
-#elif defined(_MSC_VER)
+#elif defined(FLB_SYSTEM_WINDOWS)
     int len;
 
-    h = _get_osfhandle(file->fd);
+    h = (HANDLE) _get_osfhandle(file->fd);
     if (h == INVALID_HANDLE_VALUE) {
         flb_errno();
         flb_free(buf);
@@ -1261,7 +1344,7 @@ int flb_tail_file_purge(struct flb_input_instance *ins,
             if (ret == 0) {
                 flb_plg_debug(ctx->ins,
                               "inode=%"PRIu64" purge rotated file %s " \
-                              "(offset=%lu / size = %"PRIu64")",
+                              "(offset=%"PRId64" / size = %"PRIu64")",
                               file->inode, file->name, file->offset, st.st_size);
                 if (file->pending_bytes > 0 && flb_input_buf_paused(ins)) {
                     flb_plg_warn(ctx->ins, "purged rotated file while data "
@@ -1271,7 +1354,7 @@ int flb_tail_file_purge(struct flb_input_instance *ins,
             }
             else {
                 flb_plg_debug(ctx->ins,
-                              "inode=%"PRIu64" purge rotated file %s (offset=%lu)",
+                              "inode=%"PRIu64" purge rotated file %s (offset=%"PRId64")",
                               file->inode, file->name, file->offset);
             }
 
