@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -23,8 +23,8 @@
 
 #include "tail_config.h"
 
-int flb_tail_scan(const char *path, struct flb_tail_config *ctx);
-int flb_tail_scan_callback(struct flb_input_instance *i_ins,
+int flb_tail_scan(struct mk_list *path, struct flb_tail_config *ctx);
+int flb_tail_scan_callback(struct flb_input_instance *ins,
                            struct flb_config *config, void *context);
 
 #endif
