@@ -267,6 +267,8 @@ struct flb_http_client *request_do(struct flb_aws_client *aws_client,
     int normalize_uri;
     struct flb_aws_header header;
     struct flb_http_client *c = NULL;
+    flb_sds_t tmp;
+    flb_sds_t user_agent_prefix;
 
     u_conn = flb_upstream_conn_get(aws_client->upstream);
     if (!u_conn) {
@@ -296,8 +298,27 @@ struct flb_http_client *request_do(struct flb_aws_client *aws_client,
     }
 
     /* Add AWS Fluent Bit user agent */
-    ret = flb_http_add_header(c, "User-Agent", 10,
-                              "aws-fluent-bit-plugin", 21);
+    if (aws_client->extra_user_agent == NULL) {
+        ret = flb_http_add_header(c, "User-Agent", 10,
+                                  "aws-fluent-bit-plugin", 21);
+    } 
+    else {
+        user_agent_prefix = flb_sds_create_size(64);
+        tmp = flb_sds_printf(&user_agent_prefix, "aws-fluent-bit-plugin-%s", 
+                             aws_client->extra_user_agent);
+        if (!tmp) {
+            flb_errno();
+            flb_sds_destroy(user_agent_prefix);
+            flb_error("[aws_client] failed to fetch user agent");
+            goto error;
+        }
+        user_agent_prefix = tmp;
+
+        ret = flb_http_add_header(c, "User-Agent", 10, user_agent_prefix, 
+                                  flb_sds_len(user_agent_prefix));
+        flb_sds_destroy(user_agent_prefix);
+    }
+    
     if (ret < 0) {
         if (aws_client->debug_only == FLB_TRUE) {
             flb_debug("[aws_client] failed to add header to request");
